@@ -19,6 +19,11 @@
 
 root_dir=$(pwd)
 
+# If we're running from the python subdirectory, adjust the root_dir
+if [[ "$(basename "$root_dir")" == "python" ]]; then
+    root_dir=$(dirname "$root_dir")
+fi
+
 echo $root_dir
 
 python_script_path=$root_dir/python/flink_agents/plan/tests/compatibility
@@ -37,4 +42,4 @@ function test_create_python_option_from_java_option {
   rm -f $json_path
 }
 
-test_create_python_option_from_java_option
\ No newline at end of file
+test_create_python_option_from_java_option
