@@ -18,6 +18,7 @@
 package org.apache.flink.agents.examples;
 
 import org.apache.flink.agents.api.AgentsExecutionEnvironment;
+import org.apache.flink.agents.api.agents.AgentExecutionOptions;
 import org.apache.flink.agents.api.resource.ResourceType;
 import org.apache.flink.agents.examples.agents.CustomTypesAndResources;
 import org.apache.flink.agents.examples.agents.ReviewAnalysisAgent;
@@ -55,6 +56,9 @@ public class WorkflowSingleAgentExample {
         AgentsExecutionEnvironment agentsEnv =
                 AgentsExecutionEnvironment.getExecutionEnvironment(env);
 
+        // limit async request to avoid overwhelming ollama server
+        agentsEnv.getConfig().set(AgentExecutionOptions.NUM_ASYNC_THREADS, 2);
+
         // Add Ollama chat model connection to be used by the ReviewAnalysisAgent.
         agentsEnv.addResource(
                 "ollamaChatModelConnection",
