@@ -15,15 +15,16 @@
 #  See the License for the specific language governing permissions and
 # limitations under the License.
 #################################################################################
-import importlib
-import json
-from typing import Any, cast
+from typing import cast
 
-from pydantic import BaseModel, ConfigDict, model_serializer, model_validator
+from pydantic import (
+    BaseModel,
+)
 from pyflink.common import Row
-from pyflink.common.typeinfo import BasicType, BasicTypeInfo, RowTypeInfo
+from pyflink.common.typeinfo import RowTypeInfo
 
-from flink_agents.api.agent import Agent
+from flink_agents.api.agents.agent import STRUCTURED_OUTPUT, Agent
+from flink_agents.api.agents.types import OutputSchema
 from flink_agents.api.chat_message import ChatMessage, MessageRole
 from flink_agents.api.decorators import action
 from flink_agents.api.events.chat_event import ChatRequestEvent, ChatResponseEvent
@@ -38,51 +39,6 @@ _DEFAULT_USER_PROMPT = "_default_user_prompt"
 _OUTPUT_SCHEMA = "_output_schema"
 
 
-class OutputSchema(BaseModel):
-    """Util class to help serialize and deserialize output schema json."""
-
-    model_config = ConfigDict(arbitrary_types_allowed=True)
-    output_schema: type[BaseModel] | RowTypeInfo
-
-    @model_serializer
-    def __custom_serializer(self) -> dict[str, Any]:
-        if isinstance(self.output_schema, RowTypeInfo):
-            data = {
-                "output_schema": {
-                    "names": self.output_schema.get_field_names(),
-                    "types": [
-                        type._basic_type.value
-                        for type in self.output_schema.get_field_types()
-                    ],
-                },
-            }
-        else:
-            data = {
-                "output_schema": {
-                    "module": self.output_schema.__module__,
-                    "class": self.output_schema.__name__,
-                }
-            }
-        return data
-
-    @model_validator(mode="before")
-    def __custom_deserialize(self) -> "OutputSchema":
-        output_schema = self["output_schema"]
-        if isinstance(output_schema, dict):
-            if "names" in output_schema:
-                self["output_schema"] = RowTypeInfo(
-                    field_types=[
-                        BasicTypeInfo(BasicType(type))
-                        for type in output_schema["types"]
-                    ],
-                    field_names=output_schema["names"],
-                )
-            else:
-                module = importlib.import_module(output_schema["module"])
-                self["output_schema"] = getattr(module, output_schema["class"])
-        return self
-
-
 class ReActAgent(Agent):
     """Built-in implementation of ReAct agent which is based on the function
     call ability of llm.
@@ -134,7 +90,7 @@ class ReActAgent(Agent):
                     tools=["notify_shipping_manager"],
                 ),
                 prompt=prompt,
-                output_schema=OutputData
+                output_schema=OutputData,
             )
     """
 
@@ -160,7 +116,7 @@ class ReActAgent(Agent):
             response format of llm, and add output parser according to the schema.
         """
         super().__init__()
-        self.add_resource(_DEFAULT_CHAT_MODEL, chat_model)
+        self.add_resource(_DEFAULT_CHAT_MODEL, ResourceType.CHAT_MODEL, chat_model)
 
         if output_schema:
             if isinstance(output_schema, type) and issubclass(output_schema, BaseModel):
@@ -179,13 +135,12 @@ class ReActAgent(Agent):
             self._resources[ResourceType.PROMPT][_DEFAULT_USER_PROMPT] = prompt
 
         self.add_action(
-            name="stop_action",
-            events=[ChatResponseEvent],
-            func=self.stop_action,
+            name="start_action",
+            events=[InputEvent],
+            func=self.start_action,
             output_schema=OutputSchema(output_schema=output_schema),
         )
 
-    @action(InputEvent)
     @staticmethod
     def start_action(event: InputEvent, ctx: RunnerContext) -> None:
         """Start action to format user input and send chat request event."""
@@ -232,30 +187,25 @@ class ReActAgent(Agent):
             instruct = schema_prompt.format_messages()
             usr_msgs = instruct + usr_msgs
 
+        output_schema = ctx.get_action_config_value(key="output_schema")
+
         ctx.send_event(
             ChatRequestEvent(
                 model=_DEFAULT_CHAT_MODEL,
                 messages=usr_msgs,
+                output_schema=output_schema,
             )
         )
 
+    @action(ChatResponseEvent)
     @staticmethod
     def stop_action(event: ChatResponseEvent, ctx: RunnerContext) -> None:
         """Stop action to output result."""
-        output = event.response.content
+        response = event.response
+
+        if STRUCTURED_OUTPUT in response.extra_args:
+            output = response.extra_args[STRUCTURED_OUTPUT]
+        else:
+            output = response.content
 
-        # parse llm response to target schema.
-        # TODO: config error handle strategy by configuration.
-        output_schema = ctx.get_action_config_value(key="output_schema")
-        if output_schema:
-            output_schema = output_schema.output_schema
-            output = json.loads(output.strip())
-            if isinstance(output_schema, type) and issubclass(output_schema, BaseModel):
-                output = output_schema.model_validate(output)
-            elif isinstance(output_schema, RowTypeInfo):
-                field_names = output_schema.get_field_names()
-                values = {}
-                for field_name in field_names:
-                    values[field_name] = output[field_name]
-                output = Row(**values)
         ctx.send_event(OutputEvent(output=output))
