@@ -21,21 +21,31 @@ import org.apache.flink.agents.api.Event;
 import org.apache.flink.agents.api.EventContext;
 import org.apache.flink.agents.api.InputEvent;
 import org.apache.flink.agents.api.OutputEvent;
+import org.apache.flink.agents.api.agents.AgentExecutionOptions;
 import org.apache.flink.agents.api.context.MemoryUpdate;
 import org.apache.flink.agents.api.listener.EventListener;
 import org.apache.flink.agents.api.logger.EventLogger;
 import org.apache.flink.agents.api.logger.EventLoggerConfig;
 import org.apache.flink.agents.api.logger.EventLoggerFactory;
 import org.apache.flink.agents.api.logger.EventLoggerOpenParams;
+import org.apache.flink.agents.api.resource.Resource;
+import org.apache.flink.agents.api.resource.ResourceType;
 import org.apache.flink.agents.plan.AgentPlan;
 import org.apache.flink.agents.plan.JavaFunction;
 import org.apache.flink.agents.plan.PythonFunction;
 import org.apache.flink.agents.plan.actions.Action;
+import org.apache.flink.agents.plan.resourceprovider.PythonResourceProvider;
 import org.apache.flink.agents.runtime.actionstate.ActionState;
 import org.apache.flink.agents.runtime.actionstate.ActionStateStore;
 import org.apache.flink.agents.runtime.actionstate.KafkaActionStateStore;
+import org.apache.flink.agents.runtime.async.ContinuationActionExecutor;
+import org.apache.flink.agents.runtime.async.ContinuationContext;
+import org.apache.flink.agents.runtime.context.ActionStatePersister;
+import org.apache.flink.agents.runtime.context.JavaRunnerContextImpl;
 import org.apache.flink.agents.runtime.context.RunnerContextImpl;
+import org.apache.flink.agents.runtime.env.EmbeddedPythonEnvironment;
 import org.apache.flink.agents.runtime.env.PythonEnvironmentManager;
+import org.apache.flink.agents.runtime.eventlog.FileEventLogger;
 import org.apache.flink.agents.runtime.memory.CachedMemoryStore;
 import org.apache.flink.agents.runtime.memory.MemoryObjectImpl;
 import org.apache.flink.agents.runtime.metrics.BuiltInMetrics;
@@ -44,7 +54,9 @@ import org.apache.flink.agents.runtime.operator.queue.SegmentedQueue;
 import org.apache.flink.agents.runtime.python.context.PythonRunnerContextImpl;
 import org.apache.flink.agents.runtime.python.event.PythonEvent;
 import org.apache.flink.agents.runtime.python.operator.PythonActionTask;
+import org.apache.flink.agents.runtime.python.utils.JavaResourceAdapter;
 import org.apache.flink.agents.runtime.python.utils.PythonActionExecutor;
+import org.apache.flink.agents.runtime.python.utils.PythonResourceAdapterImpl;
 import org.apache.flink.agents.runtime.utils.EventUtil;
 import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.api.common.operators.MailboxExecutor;
@@ -60,21 +72,24 @@ import org.apache.flink.runtime.state.StateInitializationContext;
 import org.apache.flink.runtime.state.StateSnapshotContext;
 import org.apache.flink.runtime.state.VoidNamespace;
 import org.apache.flink.runtime.state.VoidNamespaceSerializer;
-import org.apache.flink.shaded.jackson2.com.fasterxml.jackson.databind.ObjectMapper;
+import org.apache.flink.streaming.api.graph.StreamConfig;
 import org.apache.flink.streaming.api.operators.AbstractStreamOperator;
 import org.apache.flink.streaming.api.operators.BoundedOneInput;
 import org.apache.flink.streaming.api.operators.ChainingStrategy;
 import org.apache.flink.streaming.api.operators.OneInputStreamOperator;
+import org.apache.flink.streaming.api.operators.Output;
 import org.apache.flink.streaming.api.operators.StreamingRuntimeContext;
 import org.apache.flink.streaming.api.watermark.Watermark;
 import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
 import org.apache.flink.streaming.runtime.tasks.ProcessingTimeService;
+import org.apache.flink.streaming.runtime.tasks.StreamTask;
 import org.apache.flink.streaming.runtime.tasks.mailbox.MailboxExecutorImpl;
 import org.apache.flink.streaming.runtime.tasks.mailbox.MailboxProcessor;
 import org.apache.flink.types.Row;
 import org.apache.flink.util.ExceptionUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import pemja.core.PythonInterpreter;
 
 import java.lang.reflect.Field;
 import java.util.ArrayList;
@@ -84,6 +99,8 @@ import java.util.Map;
 import java.util.Optional;
 
 import static org.apache.flink.agents.api.configuration.AgentConfigOptions.ACTION_STATE_STORE_BACKEND;
+import static org.apache.flink.agents.api.configuration.AgentConfigOptions.BASE_LOG_DIR;
+import static org.apache.flink.agents.api.configuration.AgentConfigOptions.JOB_IDENTIFIER;
 import static org.apache.flink.agents.runtime.actionstate.ActionStateStore.BackendType.KAFKA;
 import static org.apache.flink.agents.runtime.utils.StateUtil.*;
 import static org.apache.flink.util.Preconditions.checkState;
@@ -99,7 +116,7 @@ import static org.apache.flink.util.Preconditions.checkState;
  * and the resulting output event is collected for further processing.
  */
 public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT>
-        implements OneInputStreamOperator<IN, OUT>, BoundedOneInput {
+        implements OneInputStreamOperator<IN, OUT>, BoundedOneInput, ActionStatePersister {
 
     private static final long serialVersionUID = 1L;
 
@@ -115,11 +132,26 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
 
     private transient StreamRecord<OUT> reusedStreamRecord;
 
+    private transient MapState<String, MemoryObjectImpl.MemoryItem> sensoryMemState;
+
     private transient MapState<String, MemoryObjectImpl.MemoryItem> shortTermMemState;
 
+    private transient PythonEnvironmentManager pythonEnvironmentManager;
+
+    private transient PythonInterpreter pythonInterpreter;
+
     // PythonActionExecutor for Python actions
     private transient PythonActionExecutor pythonActionExecutor;
 
+    // RunnerContext for Python actions
+    private transient PythonRunnerContextImpl pythonRunnerContext;
+
+    // PythonResourceAdapter for Python resources in Java actions
+    private transient PythonResourceAdapterImpl pythonResourceAdapter;
+
+    // PythonResourceAdapter for Java resources in Python actions or Python resources
+    private transient JavaResourceAdapter javaResourceAdapter;
+
     private transient FlinkAgentsMetricGroupImpl metricGroup;
 
     private transient BuiltInMetrics builtInMetrics;
@@ -128,6 +160,9 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
 
     private final transient MailboxExecutor mailboxExecutor;
 
+    // RunnerContext for Java Actions
+    private transient RunnerContextImpl runnerContext;
+
     // We need to check whether the current thread is the mailbox thread using the mailbox
     // processor.
     // TODO: This is a temporary workaround. In the future, we should add an interface in
@@ -158,7 +193,26 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
 
     // This in memory map keep track of the runner context for the async action task that having
     // been finished
-    private final transient Map<ActionTask, RunnerContextImpl> actionTaskRunnerContexts;
+    private final transient Map<ActionTask, RunnerContextImpl.MemoryContext>
+            actionTaskMemoryContexts;
+
+    // This in memory map keeps track of the durable execution context for async action tasks
+    // that have not been finished, allowing recovery of currentCallIndex across invocations
+    private final transient Map<ActionTask, RunnerContextImpl.DurableExecutionContext>
+            actionTaskDurableContexts;
+
+    private final transient Map<ActionTask, ContinuationContext> continuationContexts;
+
+    private final transient Map<ActionTask, String> pythonAwaitableRefs;
+
+    // Each job can only have one identifier and this identifier must be consistent across restarts.
+    // We cannot use job id as the identifier here because user may change job id by
+    // creating a savepoint, stop the job and then resume from savepoint.
+    // We use this identifier to control the visibility for long-term memory.
+    // Inspired by Apache Paimon.
+    private transient String jobIdentifier;
+
+    private transient ContinuationActionExecutor continuationActionExecutor;
 
     public ActionExecutionOperator(
             AgentPlan agentPlan,
@@ -169,19 +223,37 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
         this.agentPlan = agentPlan;
         this.inputIsJava = inputIsJava;
         this.processingTimeService = processingTimeService;
-        this.chainingStrategy = ChainingStrategy.ALWAYS;
         this.mailboxExecutor = mailboxExecutor;
-        this.eventLogger = EventLoggerFactory.createLogger(EventLoggerConfig.builder().build());
+        this.eventLogger = createEventLogger(agentPlan);
         this.eventListeners = new ArrayList<>();
         this.actionStateStore = actionStateStore;
         this.checkpointIdToSeqNums = new HashMap<>();
-        this.actionTaskRunnerContexts = new HashMap<>();
+        this.actionTaskMemoryContexts = new HashMap<>();
+        this.actionTaskDurableContexts = new HashMap<>();
+        this.continuationContexts = new HashMap<>();
+        this.pythonAwaitableRefs = new HashMap<>();
+        OperatorUtils.setChainStrategy(this, ChainingStrategy.ALWAYS);
+    }
+
+    @Override
+    public void setup(
+            StreamTask<?, ?> containingTask,
+            StreamConfig config,
+            Output<StreamRecord<OUT>> output) {
+        super.setup(containingTask, config, output);
     }
 
     @Override
     public void open() throws Exception {
         super.open();
         reusedStreamRecord = new StreamRecord<>(null);
+        // init sensoryMemState
+        MapStateDescriptor<String, MemoryObjectImpl.MemoryItem> sensoryMemStateDescriptor =
+                new MapStateDescriptor<>(
+                        "sensoryMemory",
+                        TypeInformation.of(String.class),
+                        TypeInformation.of(MemoryObjectImpl.MemoryItem.class));
+        sensoryMemState = getRuntimeContext().getMapState(sensoryMemStateDescriptor);
 
         // init shortTermMemState
         MapStateDescriptor<String, MemoryObjectImpl.MemoryItem> shortTermMemStateDescriptor =
@@ -196,13 +268,7 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
 
         keySegmentQueue = new SegmentedQueue();
 
-        // init the action state store with proper implementation
-        if (actionStateStore == null
-                && KAFKA.getType()
-                        .equalsIgnoreCase(agentPlan.getConfig().get(ACTION_STATE_STORE_BACKEND))) {
-            LOG.info("Using Kafka as backend of action state store.");
-            actionStateStore = new KafkaActionStateStore(agentPlan.getConfig());
-        }
+        maybeInitActionStateStore();
 
         if (actionStateStore != null) {
             // init recovery marker state for recovery marker persistence
@@ -242,8 +308,13 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
                                 new ListStateDescriptor<>(
                                         "currentProcessingKeys", TypeInformation.of(Object.class)));
 
-        // init PythonActionExecutor
-        initPythonActionExecutor();
+        // init PythonActionExecutor and PythonResourceAdapter
+        initPythonEnvironment();
+
+        // init executor for Java async execution
+        continuationActionExecutor =
+                new ContinuationActionExecutor(
+                        agentPlan.getConfig().get(AgentExecutionOptions.NUM_ASYNC_THREADS));
 
         mailboxProcessor = getMailboxProcessor();
 
@@ -386,7 +457,7 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
         }
 
         // 2. Invoke the action task.
-        createAndSetRunnerContext(actionTask);
+        createAndSetRunnerContext(actionTask, key);
 
         long sequenceNumber = sequenceNumberKState.value();
         boolean isFinished;
@@ -394,23 +465,53 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
         Optional<ActionTask> generatedActionTaskOpt = Optional.empty();
         ActionState actionState =
                 maybeGetActionState(key, sequenceNumber, actionTask.action, actionTask.event);
-        if (actionState != null) {
+
+        // Check if action is already completed
+        if (actionState != null && actionState.isCompleted()) {
+            // Action has completed, skip execution and replay memory/events
+            LOG.debug(
+                    "Skipping already completed action: {} for key: {}",
+                    actionTask.action.getName(),
+                    key);
             isFinished = true;
             outputEvents = actionState.getOutputEvents();
-            for (MemoryUpdate memoryUpdate : actionState.getMemoryUpdates()) {
+            for (MemoryUpdate memoryUpdate : actionState.getShortTermMemoryUpdates()) {
                 actionTask
                         .getRunnerContext()
                         .getShortTermMemory()
                         .set(memoryUpdate.getPath(), memoryUpdate.getValue());
             }
-        } else {
-            maybeInitActionState(key, sequenceNumber, actionTask.action, actionTask.event);
-            ActionTask.ActionTaskResult actionTaskResult = actionTask.invoke();
 
-            // We remove the RunnerContext of the action task from the map after it is finished. The
-            // RunnerContext will be added later if the action task has a generated action task,
-            // meaning it is not finished.
-            actionTaskRunnerContexts.remove(actionTask);
+            for (MemoryUpdate memoryUpdate : actionState.getSensoryMemoryUpdates()) {
+                actionTask
+                        .getRunnerContext()
+                        .getSensoryMemory()
+                        .set(memoryUpdate.getPath(), memoryUpdate.getValue());
+            }
+        } else {
+            // Initialize ActionState if not exists, or use existing one for recovery
+            if (actionState == null) {
+                maybeInitActionState(key, sequenceNumber, actionTask.action, actionTask.event);
+                actionState =
+                        maybeGetActionState(
+                                key, sequenceNumber, actionTask.action, actionTask.event);
+            }
+
+            // Set up durable execution context for fine-grained recovery
+            setupDurableExecutionContext(actionTask, actionState);
+
+            ActionTask.ActionTaskResult actionTaskResult =
+                    actionTask.invoke(
+                            getRuntimeContext().getUserCodeClassLoader(),
+                            this.pythonActionExecutor);
+
+            // We remove the contexts from the map after the task is processed. They will be added
+            // back later if the action task has a generated action task, meaning it is not
+            // finished.
+            actionTaskMemoryContexts.remove(actionTask);
+            actionTaskDurableContexts.remove(actionTask);
+            continuationContexts.remove(actionTask);
+            pythonAwaitableRefs.remove(actionTask);
             maybePersistTaskResult(
                     key,
                     sequenceNumber,
@@ -443,15 +544,38 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
             // execution.
             ActionTask generatedActionTask = generatedActionTaskOpt.get();
 
-            // If the action task is not finished, we keep the runner context in the memory for the
+            // If the action task is not finished, we keep the contexts in memory for the
             // next generated ActionTask to be invoked.
-            actionTaskRunnerContexts.put(generatedActionTask, actionTask.getRunnerContext());
+            actionTaskMemoryContexts.put(
+                    generatedActionTask, actionTask.getRunnerContext().getMemoryContext());
+            RunnerContextImpl.DurableExecutionContext durableContext =
+                    actionTask.getRunnerContext().getDurableExecutionContext();
+            if (durableContext != null) {
+                actionTaskDurableContexts.put(generatedActionTask, durableContext);
+            }
+            if (actionTask.getRunnerContext() instanceof JavaRunnerContextImpl) {
+                continuationContexts.put(
+                        generatedActionTask,
+                        ((JavaRunnerContextImpl) actionTask.getRunnerContext())
+                                .getContinuationContext());
+            }
+            if (actionTask.getRunnerContext() instanceof PythonRunnerContextImpl) {
+                String awaitableRef =
+                        ((PythonRunnerContextImpl) actionTask.getRunnerContext())
+                                .getPythonAwaitableRef();
+                if (awaitableRef != null) {
+                    pythonAwaitableRefs.put(generatedActionTask, awaitableRef);
+                }
+            }
 
             actionTasksKState.add(generatedActionTask);
         }
 
         // 3. Process the next InputEvent or next action task
         if (currentInputEventFinished) {
+            // Clean up sensory memory when a single run finished.
+            actionTask.getRunnerContext().clearSensoryMemory();
+
             // Once all sub-events and actions related to the current InputEvent are completed,
             // we can proceed to process the next InputEvent.
             int removedCount = removeFromListState(currentProcessingKeysOpState, key);
@@ -477,17 +601,37 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
         }
     }
 
-    private void initPythonActionExecutor() throws Exception {
+    private Resource getResource(String name, ResourceType type) {
+        try {
+            return agentPlan.getResource(name, type);
+        } catch (Exception e) {
+            throw new RuntimeException(e);
+        }
+    }
+
+    private void initPythonEnvironment() throws Exception {
         boolean containPythonAction =
                 agentPlan.getActions().values().stream()
                         .anyMatch(action -> action.getExec() instanceof PythonFunction);
-        if (containPythonAction) {
-            LOG.debug("Begin initialize PythonActionExecutor.");
+
+        boolean containPythonResource =
+                agentPlan.getResourceProviders().values().stream()
+                        .anyMatch(
+                                resourceProviderMap ->
+                                        resourceProviderMap.values().stream()
+                                                .anyMatch(
+                                                        resourceProvider ->
+                                                                resourceProvider
+                                                                        instanceof
+                                                                        PythonResourceProvider));
+
+        if (containPythonAction || containPythonResource) {
+            LOG.debug("Begin initialize PythonEnvironmentManager.");
             PythonDependencyInfo dependencyInfo =
                     PythonDependencyInfo.create(
                             getExecutionConfig().toConfiguration(),
                             getRuntimeContext().getDistributedCache());
-            PythonEnvironmentManager pythonEnvironmentManager =
+            pythonEnvironmentManager =
                     new PythonEnvironmentManager(
                             dependencyInfo,
                             getContainingTask()
@@ -496,14 +640,53 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
                                     .getTmpDirectories(),
                             new HashMap<>(System.getenv()),
                             getRuntimeContext().getJobInfo().getJobId());
-            pythonActionExecutor =
-                    new PythonActionExecutor(
-                            pythonEnvironmentManager,
-                            new ObjectMapper().writeValueAsString(agentPlan));
-            pythonActionExecutor.open();
+            pythonEnvironmentManager.open();
+            EmbeddedPythonEnvironment env = pythonEnvironmentManager.createEnvironment();
+            pythonInterpreter = env.getInterpreter();
+            pythonRunnerContext =
+                    new PythonRunnerContextImpl(
+                            this.metricGroup,
+                            this::checkMailboxThread,
+                            this.agentPlan,
+                            this.jobIdentifier);
+
+            javaResourceAdapter = new JavaResourceAdapter(this::getResource, pythonInterpreter);
+            if (containPythonResource) {
+                initPythonResourceAdapter();
+            }
+            if (containPythonAction) {
+                initPythonActionExecutor();
+            }
         }
     }
 
+    private void initPythonActionExecutor() throws Exception {
+        pythonActionExecutor =
+                new PythonActionExecutor(
+                        pythonInterpreter,
+                        agentPlan,
+                        javaResourceAdapter,
+                        pythonRunnerContext,
+                        jobIdentifier);
+        pythonActionExecutor.open();
+    }
+
+    private void initPythonResourceAdapter() throws Exception {
+        pythonResourceAdapter =
+                new PythonResourceAdapterImpl(
+                        (String anotherName, ResourceType anotherType) -> {
+                            try {
+                                return agentPlan.getResource(anotherName, anotherType);
+                            } catch (Exception e) {
+                                throw new RuntimeException(e);
+                            }
+                        },
+                        pythonInterpreter,
+                        javaResourceAdapter);
+        pythonResourceAdapter.open();
+        agentPlan.setPythonResourceAdapter(pythonResourceAdapter);
+    }
+
     @Override
     public void endInput() throws Exception {
         waitInFlightEventsFinished();
@@ -518,15 +701,34 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
 
     @Override
     public void close() throws Exception {
+        if (runnerContext != null) {
+            try {
+                runnerContext.close();
+            } finally {
+                runnerContext = null;
+            }
+        }
         if (pythonActionExecutor != null) {
             pythonActionExecutor.close();
         }
+        if (pythonInterpreter != null) {
+            pythonInterpreter.close();
+        }
+        if (pythonEnvironmentManager != null) {
+            pythonEnvironmentManager.close();
+        }
         if (eventLogger != null) {
             eventLogger.close();
         }
         if (actionStateStore != null) {
             actionStateStore.close();
         }
+        if (runnerContext != null) {
+            runnerContext.close();
+        }
+        if (continuationActionExecutor != null) {
+            continuationActionExecutor.close();
+        }
 
         super.close();
     }
@@ -535,6 +737,8 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
     public void initializeState(StateInitializationContext context) throws Exception {
         super.initializeState(context);
 
+        maybeInitActionStateStore();
+
         if (actionStateStore != null) {
             List<Object> markers = new ArrayList<>();
 
@@ -553,8 +757,19 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
             if (recoveryMarkers != null) {
                 recoveryMarkers.forEach(markers::add);
             }
+            LOG.info("Rebuilding action state from {} recovery markers", markers.size());
             actionStateStore.rebuildState(markers);
         }
+
+        // Get job identifier from user configuration.
+        // If not configured, get from state.
+        jobIdentifier = agentPlan.getConfig().get(JOB_IDENTIFIER);
+        if (jobIdentifier == null) {
+            String initialJobIdentifier = getRuntimeContext().getJobInfo().getJobId().toString();
+            jobIdentifier =
+                    StateUtils.getSingleValueFromState(
+                            context, "identifier_state", String.class, initialJobIdentifier);
+        }
     }
 
     @Override
@@ -638,8 +853,7 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
 
     private ActionTask createActionTask(Object key, Action action, Event event) {
         if (action.getExec() instanceof JavaFunction) {
-            return new JavaActionTask(
-                    key, event, action, getRuntimeContext().getUserCodeClassLoader());
+            return new JavaActionTask(key, event, action);
         } else if (action.getExec() instanceof PythonFunction) {
             return new PythonActionTask(key, event, action);
         } else {
@@ -648,35 +862,49 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
         }
     }
 
-    private void createAndSetRunnerContext(ActionTask actionTask) {
-        if (actionTask.getRunnerContext() != null) {
-            return;
-        }
-
+    private void createAndSetRunnerContext(ActionTask actionTask, Object key) {
         RunnerContextImpl runnerContext;
-        if (actionTaskRunnerContexts.containsKey(actionTask)) {
-            runnerContext = actionTaskRunnerContexts.get(actionTask);
-        } else if (actionTask.action.getExec() instanceof JavaFunction) {
-            runnerContext =
-                    new RunnerContextImpl(
-                            new CachedMemoryStore(shortTermMemState),
-                            metricGroup,
-                            this::checkMailboxThread,
-                            agentPlan);
+        if (actionTask.action.getExec() instanceof JavaFunction) {
+            runnerContext = createOrGetRunnerContext(true);
         } else if (actionTask.action.getExec() instanceof PythonFunction) {
-            runnerContext =
-                    new PythonRunnerContextImpl(
-                            new CachedMemoryStore(shortTermMemState),
-                            metricGroup,
-                            this::checkMailboxThread,
-                            agentPlan,
-                            pythonActionExecutor);
+            runnerContext = createOrGetRunnerContext(false);
         } else {
             throw new IllegalStateException(
                     "Unsupported action type: " + actionTask.action.getExec().getClass());
         }
 
-        runnerContext.setActionName(actionTask.action.getName());
+        RunnerContextImpl.MemoryContext memoryContext;
+        if (actionTaskMemoryContexts.containsKey(actionTask)) {
+            // action task for async execution action, should retrieve intermediate results from
+            // map.
+            memoryContext = actionTaskMemoryContexts.get(actionTask);
+        } else {
+            memoryContext =
+                    new RunnerContextImpl.MemoryContext(
+                            new CachedMemoryStore(sensoryMemState),
+                            new CachedMemoryStore(shortTermMemState));
+        }
+
+        runnerContext.switchActionContext(
+                actionTask.action.getName(), memoryContext, String.valueOf(key.hashCode()));
+
+        if (runnerContext instanceof JavaRunnerContextImpl) {
+            ContinuationContext continuationContext;
+            if (continuationContexts.containsKey(actionTask)) {
+                // action task for async execution action, should retrieve intermediate results from
+                // map.
+                continuationContext = continuationContexts.get(actionTask);
+            } else {
+                continuationContext = new ContinuationContext();
+            }
+            ((JavaRunnerContextImpl) runnerContext).setContinuationContext(continuationContext);
+        }
+        if (runnerContext instanceof PythonRunnerContextImpl) {
+            // Get the awaitable ref from the transient map. After checkpoint restore, this will be
+            // null, signaling that the awaitable was lost and needs re-execution.
+            String awaitableRef = pythonAwaitableRefs.get(actionTask);
+            ((PythonRunnerContextImpl) runnerContext).setPythonAwaitableRef(awaitableRef);
+        }
         actionTask.setRunnerContext(runnerContext);
     }
 
@@ -755,14 +983,79 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
 
         ActionState actionState = actionStateStore.get(key, sequenceNum, action, event);
 
-        for (MemoryUpdate memoryUpdate : context.getAllMemoryUpdates()) {
-            actionState.addMemoryUpdate(memoryUpdate);
+        for (MemoryUpdate memoryUpdate : context.getSensoryMemoryUpdates()) {
+            actionState.addSensoryMemoryUpdate(memoryUpdate);
+        }
+
+        for (MemoryUpdate memoryUpdate : context.getShortTermMemoryUpdates()) {
+            actionState.addShortTermMemoryUpdate(memoryUpdate);
         }
 
         for (Event outputEvent : actionTaskResult.getOutputEvents()) {
             actionState.addEvent(outputEvent);
         }
+
+        // Mark the action as completed and clear call records
+        // This indicates that recovery should skip the entire action
+        actionState.markCompleted();
+
         actionStateStore.put(key, sequenceNum, action, event, actionState);
+
+        // Clear durable execution context
+        context.clearDurableExecutionContext();
+    }
+
+    /**
+     * Sets up the durable execution context for fine-grained recovery.
+     *
+     * <p>This method initializes the runner context with a {@link
+     * RunnerContextImpl.DurableExecutionContext}, which enables execute/execute_async calls to:
+     *
+     * <ul>
+     *   <li>Skip re-execution for already completed calls during recovery
+     *   <li>Persist CallRecords after each code block completion
+     * </ul>
+     */
+    private void setupDurableExecutionContext(ActionTask actionTask, ActionState actionState) {
+        if (actionStateStore == null) {
+            return;
+        }
+
+        RunnerContextImpl.DurableExecutionContext durableContext;
+        if (actionTaskDurableContexts.containsKey(actionTask)) {
+            // Reuse existing context for async action continuation
+            durableContext = actionTaskDurableContexts.get(actionTask);
+        } else {
+            // Create new context for first invocation
+            final long sequenceNumber;
+            try {
+                sequenceNumber = sequenceNumberKState.value();
+            } catch (Exception e) {
+                throw new RuntimeException("Failed to get sequence number from state", e);
+            }
+
+            durableContext =
+                    new RunnerContextImpl.DurableExecutionContext(
+                            actionTask.getKey(),
+                            sequenceNumber,
+                            actionTask.action,
+                            actionTask.event,
+                            actionState,
+                            this);
+        }
+
+        actionTask.getRunnerContext().setDurableExecutionContext(durableContext);
+    }
+
+    @Override
+    public void persist(
+            Object key, long sequenceNumber, Action action, Event event, ActionState actionState) {
+        try {
+            actionStateStore.put(key, sequenceNumber, action, event, actionState);
+        } catch (Exception e) {
+            LOG.error("Failed to persist ActionState", e);
+            throw new RuntimeException("Failed to persist ActionState", e);
+        }
     }
 
     private void maybePruneState(Object key, long sequenceNum) throws Exception {
@@ -779,6 +1072,56 @@ public class ActionExecutionOperator<IN, OUT> extends AbstractStreamOperator<OUT
         }
     }
 
+    private RunnerContextImpl createOrGetRunnerContext(Boolean isJava) {
+        if (isJava) {
+            if (runnerContext == null) {
+                if (continuationActionExecutor == null) {
+                    continuationActionExecutor =
+                            new ContinuationActionExecutor(
+                                    agentPlan
+                                            .getConfig()
+                                            .get(AgentExecutionOptions.NUM_ASYNC_THREADS));
+                }
+                runnerContext =
+                        new JavaRunnerContextImpl(
+                                this.metricGroup,
+                                this::checkMailboxThread,
+                                this.agentPlan,
+                                this.jobIdentifier,
+                                continuationActionExecutor);
+            }
+            return runnerContext;
+        } else {
+            if (pythonRunnerContext == null) {
+                pythonRunnerContext =
+                        new PythonRunnerContextImpl(
+                                this.metricGroup,
+                                this::checkMailboxThread,
+                                this.agentPlan,
+                                jobIdentifier);
+            }
+            return pythonRunnerContext;
+        }
+    }
+
+    private EventLogger createEventLogger(AgentPlan agentPlan) {
+        EventLoggerConfig.Builder loggerConfigBuilder = EventLoggerConfig.builder();
+        String baseLogDir = agentPlan.getConfig().get(BASE_LOG_DIR);
+        if (baseLogDir != null && !baseLogDir.trim().isEmpty()) {
+            loggerConfigBuilder.property(FileEventLogger.BASE_LOG_DIR_PROPERTY_KEY, baseLogDir);
+        }
+        return EventLoggerFactory.createLogger(loggerConfigBuilder.build());
+    }
+
+    private void maybeInitActionStateStore() {
+        if (actionStateStore == null
+                && KAFKA.getType()
+                        .equalsIgnoreCase(agentPlan.getConfig().get(ACTION_STATE_STORE_BACKEND))) {
+            LOG.info("Using Kafka as backend of action state store.");
+            actionStateStore = new KafkaActionStateStore(agentPlan.getConfig());
+        }
+    }
+
     /** Failed to execute Action task. */
     public static class ActionTaskExecutionException extends Exception {
         public ActionTaskExecutionException(String message, Throwable cause) {
